Package: rdbnomics
Type: Package
Title: Download DBnomics Data
Version: 0.6.4
Authors@R: c(person("Sebastien", "Galais", role = c("cre", "ctb"),
                     email = "s915.stem@gmail.com"),
             person("Thomas", "Brand", role = c("aut"),
                     email = "thomas.brand@cepremap.org"))
Description: R access to hundreds of millions data series from DBnomics API
    (<https://db.nomics.world/>).
Depends: R (>= 3.1.0)
License: AGPL-3
URL: https://git.nomics.world/dbnomics/rdbnomics
BugReports: https://git.nomics.world/dbnomics/rdbnomics/-/issues
Encoding: UTF-8
LazyData: true
Imports: curl, data.table, jsonlite
Suggests: knitr, rmarkdown, tinytest
VignetteBuilder: knitr
RoxygenNote: 7.1.1
NeedsCompilation: no
Packaged: 2020-10-25 18:23:42 UTC; sebas
Author: Sebastien Galais [cre, ctb],
  Thomas Brand [aut]
Maintainer: Sebastien Galais <s915.stem@gmail.com>
Repository: CRAN
Date/Publication: 2020-10-25 20:10:02 UTC
Built: R 4.6.0; ; 2026-04-25 21:40:19 UTC; unix
