# Generated by roxygen2: do not edit by hand

export(dbnomics)
export(rdb)
export(rdb_by_api_link)
export(rdb_datasets)
export(rdb_dimensions)
export(rdb_last_updates)
export(rdb_providers)
export(rdb_rename_xts)
export(rdb_series)
export(rdb_to_xts)
import(curl)
import(data.table)
import(jsonlite)
