% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rdbnomics.R
\docType{package}
\name{rdbnomics}
\alias{rdbnomics}
\title{Package rdbnomics}
\description{
DBnomics R client (<https://db.nomics.world/>).
}
